#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

#include "io.h"
#include "nand_public.h"
#include "nand_private.h"

static void bus_set_direction(uint8_t dir){
DDR_NAND_BUS=(dir==0)?0x00:0xff;
}

static void bus_write(uint8_t val){
PORT_NAND_BUS=val;
}

static uint8_t bus_read(void){
return PIN_NAND_BUS;
}

static uint8_t die_set_mode(int die,char mode,uint8_t parameter){
uint8_t result=0x00;
if(mode!=MODE_STANDBY && mode!=MODE_IDLE)
  bus_set_direction(mode==MODE_DATAOUT?0:1);
switch(mode){
  case MODE_STANDBY:
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_STBY_CTL1(die));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_STBY_CTL2(die));
    break;
  case MODE_IDLE:
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_IDLE_CTL1(die));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_IDLE_CTL2(die));
    break;
  case MODE_COMMAND:
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_CMD_CTL1(die,0));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_CMD_CTL2(die,0));
    bus_write(parameter);
    _delay_us(TIME_COMMAND_SET);
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_CMD_CTL1(die,1));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_CMD_CTL2(die,1));
    _delay_us(TIME_COMMAND_HOLD);
    break;
  case MODE_ADDRESS:
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_ADD_CTL1(die,0));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_ADD_CTL2(die,0));
    bus_write(parameter);
    _delay_us(TIME_ADDRESS_SET);
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_ADD_CTL1(die,1));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_ADD_CTL2(die,1));
    _delay_us(TIME_ADDRESS_HOLD);
    break;
  case MODE_DATAIN:
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_DIN_CTL1(die,0));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_DIN_CTL2(die,0));
    bus_write(parameter);
    _delay_us(TIME_DATAIN_SET);
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_DIN_CTL1(die,1));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_DIN_CTL2(die,1));
    _delay_us(TIME_DATAIN_HOLD);
    break;
  case MODE_DATAOUT:
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_DOUT_CTL1(die,0));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_DOUT_CTL2(die,0));
    _delay_us(TIME_DATAOUT_WAIT);
    result=bus_read();
    _delay_us(TIME_DATAOUT_READ);
    SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,MODE_DOUT_CTL1(die,1));
    SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,MODE_DOUT_CTL2(die,1));
    _delay_us(TIME_DATAOUT_HOLD);
    break;
  }
return result;
}

void nand_init(void){
SET_PORT(DDR_NAND_CTL1,NAND_CTL1_OMASK|NAND_CTL1_IMASK,NAND_CTL1_OMASK);
SET_PORT(DDR_NAND_CTL2,NAND_CTL2_OMASK,NAND_CTL2_OMASK);
SET_PORT(PORT_NAND_CTL1,NAND_CTL1_OMASK,(1<<NAND_CTL1_RE)|(1<<NAND_CTL1_CE));
SET_PORT(PORT_NAND_CTL2,NAND_CTL2_OMASK,(1<<NAND_CTL2_CE2)|(1<<NAND_CTL2_WE));
}

uint8_t die_is_busy(uint8_t die){
return (PIN_NAND_CTL1&(1<<((die==0)?NAND_CTL1_RB1:NAND_CTL1_RB2)))?0:1;
}

void nand_reset(void){
int i;
for(i=0;i<DIE_NB;i++){
  die_set_mode(i,MODE_COMMAND,CMD_RESET);
  while(die_is_busy(i));
  }
}

uint8_t *nand_get_std_ID(int die){
static uint8_t values[SIZE_ID_STD];
die_set_mode(die,MODE_COMMAND,CMD_READID);
die_set_mode(die,MODE_ADDRESS,ADDR_READID_STD);
_delay_us(TIME_WHR);
int i;
for(i=0;i<SIZE_ID_STD;i++) values[i]=die_set_mode(die,MODE_DATAOUT,0x00);
return values;
}

uint8_t *nand_get_ONFI_ID(int die){
static uint8_t values[SIZE_ID_ONFI];
die_set_mode(die,MODE_COMMAND,CMD_READID);
die_set_mode(die,MODE_ADDRESS,ADDR_READID_ONFI);
_delay_us(TIME_WHR);
int i;
for(i=0;i<SIZE_ID_ONFI;i++) values[i]=die_set_mode(die,MODE_DATAOUT,0x00);
return values;
}

uint8_t *nand_get_JEDEC_ID(int die){
static uint8_t values[SIZE_ID_JEDEC];
die_set_mode(die,MODE_COMMAND,CMD_READID);
die_set_mode(die,MODE_ADDRESS,ADDR_READID_JEDEC);
_delay_us(TIME_WHR);
int i;
for(i=0;i<SIZE_ID_JEDEC;i++) values[i]=die_set_mode(die,MODE_DATAOUT,0x00);
return values;
}
