/* General flash attributes */

#define DIE_NB			2

/* Definition for NAND pins */

#define DDR_NAND_BUS		DDRD
#define PORT_NAND_BUS		PORTD
#define PIN_NAND_BUS		PIND
#define NAND_BUS_0		0
#define NAND_BUS_1		1
#define NAND_BUS_2		2
#define NAND_BUS_3		3
#define NAND_BUS_4		4
#define NAND_BUS_5		5
#define NAND_BUS_6		6
#define NAND_BUS_7		7

#define DDR_NAND_CTL1		DDRB
#define PORT_NAND_CTL1		PORTB
#define PIN_NAND_CTL1		PINB
#define NAND_CTL1_RB2		4
#define NAND_CTL1_RB1		5
#define NAND_CTL1_RE		6
#define NAND_CTL1_CE		7
#define NAND_CTL1_OMASK		((1<<NAND_CTL1_RE)|(1<<NAND_CTL1_CE))
#define NAND_CTL1_IMASK		((1<<NAND_CTL1_RB2)|(1<<NAND_CTL1_RB1))

#define DDR_NAND_CTL2		DDRC
#define PORT_NAND_CTL2		PORTC
#define PIN_NAND_CTL2		PINC
#define NAND_CTL2_CE2		2
#define NAND_CTL2_CLE		4
#define NAND_CTL2_ALE		5
#define NAND_CTL2_WE		6
#define NAND_CTL2_OMASK		((1<<NAND_CTL2_CE2)|(1<<NAND_CTL2_CLE)|(1<<NAND_CTL2_ALE)|(1<<NAND_CTL2_WE))

/* Definitions for flash modes */

#define MODE_STANDBY		0
#define MODE_IDLE		1
#define MODE_COMMAND		2
#define MODE_ADDRESS		3
#define MODE_DATAIN		4
#define MODE_DATAOUT		5

/* Definitions for commands */

#define CMD_RESET		0xff
#define CMD_READID		0x90
#define CMD_READ_BEGIN		0x00
#define CMD_READ_GO		0x30
#define CMD_WRITE_BEGIN		0x80
#define CMD_WRITE_GO		0x10
#define CMD_ERASE		0x60
#define CMD_ERASE_BLOCK		0xd0

#define ADDR_READID_STD		0x00
#define ADDR_READID_ONFI	0x20
#define ADDR_READID_JEDEC	0x40

/* Definitions for timings */

#define TIME_ADL		0.200
#define TIME_ALS		0.050
#define TIME_ALH		0.020
#define TIME_CS			0.070
#define TIME_CH			0.020
#define TIME_CLS		0.050
#define TIME_CLH		0.020
#define TIME_DS			0.040
#define TIME_DH			0.020
#define TIME_REA		0.040
#define TIME_REH		0.030
#define TIME_RP			0.050
#define TIME_RR			0.020
#define TIME_WB			0.100
#define TIME_WHR		0.120
#define TIME_WP			0.050
#define TIME_WH			0.030

#define TIME_COMMAND_SET	max(TIME_ALS,max(TIME_CLS,max(TIME_CS,max(TIME_DS,TIME_WP))))
#define TIME_COMMAND_HOLD	max(TIME_ALH,max(TIME_CLH,max(TIME_CH,TIME_DH)))

#define TIME_ADDRESS_SET	TIME_COMMAND_SET
#define TIME_ADDRESS_HOLD	max(TIME_ALH,max(TIME_DH,TIME_WH))

#define TIME_DATAIN_SET		max(TIME_ALS,max(TIME_DS,TIME_WP))
#define TIME_DATAIN_HOLD	max(TIME_CLH,max(TIME_CH,max(TIME_DH,TIME_WH)))

#define TIME_DATAOUT_WAIT	TIME_REA
#define TIME_DATAOUT_READ	(TIME_RP-TIME_REA)
#define TIME_DATAOUT_HOLD	TIME_REH

/* Macros for NAND operations */

#define MODE_STBY_CTL1(die)	((die==0)?(1<<NAND_CTL1_CE):(PIN_NAND_CTL1&(1<<NAND_CTL1_CE)))
#define MODE_STBY_CTL2(die)	((die==1)?(1<<NAND_CTL2_CE2):(PIN_NAND_CTL2&(1<<NAND_CTL2_CE2)))

#define MODE_IDLE_CTL1(die)	((1<<NAND_CTL1_RE)|((die!=0)?(1<<NAND_CTL1_CE):0))
#define MODE_IDLE_CTL2(die)	((1<<NAND_CTL2_WE)|((die!=1)?(1<<NAND_CTL2_CE2):0))

#define MODE_CMD_CTL1(die,lvl)	(((die!=0)?(1<<NAND_CTL1_CE):0)|(1<<NAND_CTL1_RE))
#define MODE_CMD_CTL2(die,lvl)	(((die!=1)?(1<<NAND_CTL2_CE2):0)|(lvl<<NAND_CTL2_WE)|(1<<NAND_CTL2_CLE))

#define MODE_ADD_CTL1(die,lvl)	(((die!=0)?(1<<NAND_CTL1_CE):0)|(1<<NAND_CTL1_RE))
#define MODE_ADD_CTL2(die,lvl)	(((die!=1)?(1<<NAND_CTL2_CE2):0)|(lvl<<NAND_CTL2_WE)|(1<<NAND_CTL2_ALE))

#define MODE_DIN_CTL1(die,lvl)	(((die!=0)?(1<<NAND_CTL1_CE):0)|(1<<NAND_CTL1_RE))
#define MODE_DIN_CTL2(die,lvl)	(((die!=1)?(1<<NAND_CTL2_CE2):0)|(lvl<<NAND_CTL2_WE))

#define MODE_DOUT_CTL1(die,lvl)	(((die!=0)?(1<<NAND_CTL1_CE):0)|(lvl<<NAND_CTL1_RE))
#define MODE_DOUT_CTL2(die,lvl)	(((die!=1)?(1<<NAND_CTL2_CE2):0)|(lvl<<NAND_CTL2_WE))
